import markdown

class TestUpperCaseLetterList:
    md = markdown.Markdown(extensions=['fancylists', 'sane_lists'])

    def test_start_default(self):
        text = 'A. List item\nB. List item'
        expected = '<ol type="A">\n<li>List item</li>\n<li>List item</li>\n</ol>'

        result = self.md.convert(text)

        assert result == expected


    def test_start_B(self):
        text = 'B. List item\nC. List item'
        expected = '<ol start="2" type="A">\n<li>List item</li>\n<li>List item</li>\n</ol>'

        result = self.md.convert(text)

        assert result == expected


    def test_all_A(self):
        text = 'A. List item\nA. List item\nA. List item'
        expected = '<ol type="A">\n<li>List item</li>\n<li>List item</li>\n<li>List item</li>\n</ol>'

        result = self.md.convert(text)

        assert result == expected'


    def test_start_AA(self):
        text = 'AA. List item\nAB. List item'
        expected = '<ol start="27" type="A">\n<li>List item</li>\n<li>List item</li>\n</ol>'

        result = self.md.convert(text)

        assert result == expected


    def test_start_ZZ(self):
        text = 'ZZ. List item\nAAA. List item'
        expected = '<ol start="702" type="A">\n<li>List item</li>\n<li>List item</li>\n</ol>'

        result = self.md.convert(text)

        assert result == expected



import markdown

class TestNumberedList:
    md = markdown.Markdown(extensions=['fancylists', 'sane_lists'])

    def test_start_default(self):
        text = "1. List item\n2. List item"
        expected = "<ol>\n<li>List item</li>\n<li>List item</li>\n</ol>"

        result = self.md.convert(text)

        assert result == expected


    def test_start_2(self):
        text = "2. List item\n3. List item"
        expected = "<ol start=\"2\">\n<li>List item</li>\n<li>List item</li>\n</ol>"

        result = self.md.convert(text)

        assert result == expected


    def test_all_1(self):
        text = "1. List item\n1. List item\n1. List item"
        expected = "<ol>\n<li>List item</li>\n<li>List item</li>\n<li>List item</li>\n</ol>"

        result = self.md.convert(text)

        assert result == expected



from markdown import Extension
from markdown.blockprocessors import OListProcessor
from markdown.blockparser import BlockParser
from lxml import etree
import re


def letter_index_to_number(index, reference):
    count = 0
    for letter in index:
        count *= 26
        count += ord(letter) - ord(reference) + 1
    return count


def lower_letter_to_number(letter):
    return letter_index_to_number(letter, 'a')


def upper_letter_to_number(letter):
    return letter_index_to_number(letter, 'A')


class FancylistsProcessor(OListProcessor):
    def __init__(self, parser: BlockParser):
        super().__init__(parser)

        self.RE = re.compile(r'^[ ]{0,%d}([0-9a-zA-Z]+)\.[ ]+(.*)' % (self.tab_length - 1))

        self.CHILD_RE = re.compile(r'^[ ]{0,%d}(([0-9a-zA-Z]+\.)|[*+-])[ ]+(.*)' %
                                   (self.tab_length - 1))

        self.INDENT_RE = re.compile(r'^[ ]{%d,%d}(([0-9a-zA-Z]+\.)|[*+-])[ ]+.*' %
                                    (self.tab_length, self.tab_length * 2 - 1))

        self.TYPE = "1"

        self.LAZY_OL = False


    def run(self, parent: etree.Element, blocks: list[str]) -> None:
        super().run(parent, blocks)

        lst = parent.find('.//ol')

        if self.TYPE != "1":
           lst.set('type', self.TYPE)


    def get_items(self, block: str) -> list[str]:
        """ Break a block into list items. """
        items = []
        for line in block.split('\n'):
            m = self.CHILD_RE.match(line)
            if m:
                # This is a new list item
                # Check first item for the start index
                if not items and self.TAG == 'ol':
                    # Detect the integer value of first list item
                    # INDEX_RE = re.compile(r'([0-9a-zA-Z]+)')
                    INDEX_RE = re.compile(r'(?P<number>\d+)|(?P<lower_letter>[a-z]+)|(?P<upper_letter>[A-Z]+)')
                    index_match = INDEX_RE.match(m.group(1))

                    if index_match:
                        if index_match.group('number'):
                            self.TYPE = "1"
                            self.STARTSWITH = index_match.group()
                        elif index_match.group('lower_letter'):
                            self.TYPE = "a"
                            self.STARTSWITH = str(lower_letter_to_number(index_match.group()))
                        elif index_match.group('upper_letter'):
                            self.TYPE = "A"
                            self.STARTSWITH = str(upper_letter_to_number(index_match.group()))

                # Append to the list
                items.append(m.group(3))
            elif self.INDENT_RE.match(line):
                # This is an indented (possibly nested) item.
                if items[-1].startswith(' '*self.tab_length):
                    # Previous item was indented. Append to that item.
                    items[-1] = '{}\n{}'.format(items[-1], line)
                else:
                    items.append(line)
            else:
                # This is another line of previous item. Append to that item.
                items[-1] = '{}\n{}'.format(items[-1], line)
        return items


class FancylistsExtension(Extension):
    def extendMarkdown(self, md):
        md.parser.blockprocessors.register(FancylistsProcessor(md.parser), 'fancylist', 50)


def makeExtension(**kwargs):
    """Return extension."""

    return FancylistsExtension(**kwargs)



from fancylists.fancylists import letter_index_to_number

def test_single_lowercase_letter():
    assert letter_index_to_number('a', 'a') == 1
    assert letter_index_to_number('b', 'a') == 2
    assert letter_index_to_number('z', 'a') == 26

def test_multiple_lowercase_letters():
    assert letter_index_to_number('aa', 'a') == 27
    assert letter_index_to_number('ab', 'a') == 28
    assert letter_index_to_number('az', 'a') == 52
    assert letter_index_to_number('ba', 'a') == 53
    assert letter_index_to_number('zz', 'a') == 702

def test_single_uppercase_letter():
    assert letter_index_to_number('A', 'A') == 1
    assert letter_index_to_number('B', 'A') == 2
    assert letter_index_to_number('Z', 'A') == 26

def test_multiple_uppercase_letters():
    assert letter_index_to_number('AA', 'A') == 27
    assert letter_index_to_number('AB', 'A') == 28
    assert letter_index_to_number('AZ', 'A') == 52
    assert letter_index_to_number('BA', 'A') == 53
    assert letter_index_to_number('ZZ', 'A') == 702



import markdown

class TestLowerCaseLetterList:
    md = markdown.Markdown(extensions=['fancylists', 'sane_lists'])

    def test_start_default(self):
        text = 'a. List item\nb. List item'
        expected = '<ol type="a">\n<li>List item</li>\n<li>List item</li>\n</ol>'

        result = self.md.convert(text)

        assert result == expected


    def test_start_b(self):
        text = 'b. List item\nc. List item'
        expected = '<ol start="2" type="a">\n<li>List item</li>\n<li>List item</li>\n</ol>'

        result = self.md.convert(text)

        assert result == expected


    def test_all_A(self):
        text = 'a. List item\na. List item\na. List item'
        expected = '<ol type="a">\n<li>List item</li>\n<li>List item</li>\n<li>List item</li>\n</ol>'

        result = self.md.convert(text)

        assert result == expected


    def test_start_aa(self):
        text = 'aa. List item\nab. List item'
        expected = '<ol start="27" type="a">\n<li>List item</li>\n<li>List item</li>\n</ol>'

        result = self.md.convert(text)

        assert result == expected


    def test_start_zz(self):
        text = 'zz. List item\naaa. List item'
        expected = '<ol start="702" type="a">\n<li>List item</li>\n<li>List item</li>\n</ol>'

        result = self.md.convert(text)

        assert result == expected
